//zoneman.java Winter 2003
//Features:
// Show servers
// Edit servers

import java.io.*;
import java.text.*;

import java.util.Hashtable;
import java.util.Enumeration;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;

public class zoneman
{
	static final int version = 7;

	//Modules
	static loggers lm;
	static config  cfg;
	static ui      win;
	static core    ml;

	static Hashtable serverTable;
	static Hashtable descriptionTable;
	
	private JFrame frame;
	private JList  serverList;
	private JList  popList;
	private DefaultListModel serverListModel;
	private DefaultListModel popListModel;
	
	private JTextField     nameField;
	private JPasswordField passField;
	private JTextField     autoField;
	
	public zoneman(loggers lm, config cfg, ui win, core ml)
	{
		this.lm  = lm;
		this.cfg = cfg;
		this.win = win;
		this.ml  = ml;
		
		//look and feel
		try 
		{
			UIManager.setLookAndFeel(
				UIManager.getCrossPlatformLookAndFeelClassName()
				);
		}
		catch (Exception e)
		{
		}

		//initialise window
		frame = new JFrame("zone manager");
		Component contents = createComponents();
		frame.getContentPane().add(contents, BorderLayout.CENTER);

		//Finish setting up the frame, and show it.
		frame.addWindowListener(
			new WindowAdapter()
			{
 				public void windowClosing(WindowEvent e)
				{
					writeConfig("chatnet.conf");
					System.exit(0);
					//Wants to be final!?
					//ml.stage = ml.STAGE_QUIT;
				}
			}
			);
		//Use code above to save log
		//frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		//Center on screen
		frame.setLocation(64, 64);
		frame.pack();
	}

	public Component createComponents()
	{
		//Create the components to go into the frame...
		String str;
/*
|------------------------------------|
| Name  Pass                         |
|------------------------------------|
|server list              | ping     |
|                         | list     |
|                         |          |
|                         |          |
|------------------------------------|
| Auto commands                      |
|------------------------------------|
|Connect Ping New Server Delete Edit |
|------------------------------------|
*/
		//Name
		JLabel nameLabel = new JLabel("Name: ");
		str = cfg.GetStr("ChatNet", "Name");
		if (str == null) str = "chatnet";
		nameField = new JTextField(str);
		nameField.setFont(new Font("Monospaced", Font.PLAIN, 11));
		//nameField.setColumns(16);
		//nameField.setEditable(false);

		//Password
		JLabel passLabel = new JLabel("Pass: ");
		str = cfg.GetStr("ChatNet", "Pass");
		if (str == null) str = "@chatnet";
		passField = new JPasswordField(str);
		passField.setFont(new Font("Monospaced", Font.PLAIN, 11));
		//nameField.setColumns(16);
		//passField.setEditable(false);

		//Zone list
		ListListener listListener = new ListListener();
		serverListModel = new DefaultListModel();
		serverList = new JList(serverListModel);
		serverList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		serverList.setLayoutOrientation(JList.VERTICAL);
		//serverList.setVisibleRowCount(8);
		//serverList.setFixedCellWidth(256);
		serverList.setFont(new Font("Monospaced", Font.PLAIN, 11));
		//serverList.setBackground(Color.BLACK);
		//serverList.setForeground(colors[this.YELLOW]);
		//serverList.setPreferredSize(new Dimension(256, 128));
		serverList.setMaximumSize(
			new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE)
			);
		serverList.addListSelectionListener(listListener);

		popListModel = new DefaultListModel();
		popList = new JList(popListModel);
		popList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		popList.setLayoutOrientation(JList.VERTICAL);
		//popList.setVisibleRowCount(8);
		//popList.setFixedCellWidth(48);
		popList.setFont(new Font("Monospaced", Font.PLAIN, 11));
		//popList.setBackground(Color.BLACK);
		//popList.setForeground(colors[this.YELLOW]);
		popList.setPreferredSize(new Dimension(48, 128));
		popList.setMaximumSize(
			new Dimension(48, Integer.MAX_VALUE)
			);
		popList.addListSelectionListener(listListener);

		//Auto commands
		JLabel autoLabel = new JLabel("Auto: ");
		autoField = new JTextField( cfg.GetStr("ChatNet", "Auto") );
		autoField.setFont(new Font("Monospaced", Font.PLAIN, 11));
		//autoField.setEditable(false);

		//Buttons
		JButton connectButton = new JButton("Connect");
		JButton pingButton    = new JButton("Ping");
		JButton newButton     = new JButton("New Server");
		JButton deleteButton  = new JButton("Delete");
		JButton editButton    = new JButton("Edit");

		connectButton.addActionListener(new ConnectListener());
		pingButton.addActionListener(new PingListener());
		deleteButton.addActionListener(new DeleteListener());
		
		JPanel pane;
		JPanel loginPane;
		JPanel serverPane;
		JPanel autoPane;
		JPanel menuPane;
		JScrollPane scroller;
		
		//...stick them in a container
		pane = new JPanel();
		pane.setLayout(new BoxLayout(pane, BoxLayout.Y_AXIS));
		pane.setBorder(
			BorderFactory.createEmptyBorder(
				2, //Top
				2, //Left
				2, //Bottom
				2) //Right
			);
		//pane.setLayout(new BoxLayout(pane, BoxLayout.Y_AXIS));
		
		loginPane  = new JPanel();
		loginPane.setLayout(new BoxLayout(loginPane, BoxLayout.X_AXIS));
		serverPane = new JPanel();
		serverPane.setLayout(new BoxLayout(serverPane, BoxLayout.X_AXIS));
		autoPane   = new JPanel();
		autoPane.setLayout(new BoxLayout(autoPane, BoxLayout.X_AXIS));
		menuPane   = new JPanel();
		menuPane.setLayout(new BoxLayout(menuPane, BoxLayout.X_AXIS));
		
		//Add sub-panes
		//pane.add(loginPane);
		//pane.add(serverPane);
		//pane.add(autoField); //autoText
		//pane.add(menuPane);
		
		//Add controls
		//loginPane
		loginPane.add(nameLabel);
		loginPane.add(nameField);
		loginPane.add(passLabel);
		loginPane.add(passField);
		loginPane.add(Box.createHorizontalGlue());
		loginPane.setMaximumSize(
			new Dimension(Integer.MAX_VALUE, 20)
			);
		loginPane.setBorder(
			BorderFactory.createEmptyBorder(0, 0, 2, 0)
			);
		pane.add(loginPane);
		
		//serverPane
/*
		scroller = new JScrollPane(serverList,
			JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
			JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		serverPane.add(scroller);

		scroller = new JScrollPane(popList,
			JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
			JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		serverPane.add(scroller);
*/
		serverPane.add(serverList);
		//serverPane.add(Box.createHorizontalGlue());
		serverPane.add(popList);
		//serverPane.setPreferredSize(new Dimension(256, 128));
		scroller = new JScrollPane(serverPane,
			JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
			JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		pane.add(scroller);
		
		//autoText
		autoPane.add(autoLabel);
		autoPane.add(autoField);
		autoPane.setMaximumSize(
			new Dimension(Integer.MAX_VALUE, 20)
			);
		//autoPane.setBorder(
		//	BorderFactory.createEmptyBorder(2, 0, 2, 0)
		//	);
		pane.add(autoPane);
		
		//menuPane
		menuPane.add(Box.createHorizontalGlue());
		menuPane.add(connectButton);
		menuPane.add(pingButton);
		menuPane.add(newButton);
		menuPane.add(deleteButton);
		menuPane.add(editButton);
		menuPane.add(Box.createHorizontalGlue());
		menuPane.setMaximumSize(
			new Dimension(Integer.MAX_VALUE, 20)
			);
		//menuPane.setAlignmentX(Component.CENTER_ALIGNMENT);
		pane.add(menuPane);
		
		return pane;
	}

	class ListListener implements ListSelectionListener
	{
		public void valueChanged(ListSelectionEvent e)
		{
			if (e.getSource() == serverList)
			{
				popList.setSelectedIndex(
					serverList.getSelectedIndex()
					);
			}
			else
			{
				serverList.setSelectedIndex(
					popList.getSelectedIndex()
					);
			}
		}
	}

	class ConnectListener implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			//String address = cfg.GetStr("ChatNet", "Host");
			//int    port    = cfg.GetInt("ChatNet", "Port", 5002);
			//String name    = cfg.GetStr("ChatNet", "Name");
			//String pass    = cfg.GetStr("ChatNet", "Pass");
			//String arena   = cfg.GetStr("ChatNet", "Arena");
			//String auto    = cfg.GetStr("ChatNet", "Auto");
			
			//Get info for the selected server
			int index = serverList.getSelectedIndex();
			if (index < 0) return;
			String info = (String)serverTable.get("" + index);
			String[] part = info.split(",", 5);
			
			String address = getServerHost();
			int    port    = getServerPort();

			String name  = nameField.getText();
			String pass  = new String( passField.getPassword() );
			String arena = cfg.GetStr("ChatNet", "Arena");
			if (arena == null) arena = "";
			String auto  = autoField.getText();

			hide();
			win.show();
			ml.connect(address, port, name, pass, arena, auto);
			ml.stage = ml.STAGE_RUNNING;
		}
	}

	class PingListener implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			//Get info for the selected server
			int index = serverList.getSelectedIndex();
			if (index < 0) return;
			String address = getServerHost();
			int    port    = getServerPort() + 1;
			
			//Ping
			ping pinger = new ping(lm);			
			int num = pinger.pingServer(address, port);
			pinger.finish();
			
			String playercount = "" + num;
			if (num < 0) playercount = "[DOWN]";
			popListModel.set(index, playercount);
		}
	}

	class DeleteListener implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			int index = serverList.getSelectedIndex();
			if (index < 0) return;
			serverTable.remove("" + index);
			descriptionTable.remove("" + index);
			writeProfile("zone.dat");
			readProfile("zone.dat");
		}
	}
	
	public void show()
	{
		frame.setVisible(true);
		readProfile("zone.dat");
	}

	public void hide()
	{
		frame.setVisible(false);
		writeProfile("zone.dat");
		writeConfig("chatnet.conf");
	}
	
	//Server list management
	public void addServer(String name, String playercount)
	{
		serverListModel.addElement(name);
		popListModel.addElement(playercount);
		//serverList.setToolTipText(name);
		//popList.setToolTipText(playercount);
	}
	
	public void resetServer()
	{
		serverListModel = new DefaultListModel();
		serverList.setModel(serverListModel);
		
		popListModel = new DefaultListModel();
		popList.setModel(popListModel);
	}

	public String getServerName()
	{
		//Get info for the selected server
		int index = serverList.getSelectedIndex();
		if (index < 0) return null;
		String info = (String)serverTable.get("" + index);
		String[] part = info.split(",", 5);
		return part[0];
	}

	public String getServerHost()
	{
		//Get info for the selected server
		int index = serverList.getSelectedIndex();
		if (index < 0) return null;
		String info = (String)serverTable.get("" + index);
		String[] part = info.split(",", 5);
		return part[1];
	}
	
	public int getServerPort()
	{
		//Get info for the selected server
		int index = serverList.getSelectedIndex();
		if (index < 0) return 5000;
		String info = (String)serverTable.get("" + index);
		String[] part = info.split(",", 5);
		return Integer.parseInt( part[2] );
	}
	
	//Profile management
	public void readProfile(String fname)
	{
		Reader in;
		boolean bPingNow = cfg.GetInt("ChatNet", "PingOnLoad", 1) == 1;

		//Remove all entries
		resetServer();
		serverTable      = new Hashtable();
		descriptionTable = new Hashtable();

		try
		{
			in = new FileReader( fname );
		} catch(IOException e)
		{
			lm.Log("E", "<zoneman> Cannot open '" + fname + "' for reading");
			lm.Log("E", "<zoneman> " + e.getMessage());
			descriptionTable.put("0", "this is a test zone for asss.");
			serverTable.put("0", "asss test zone,216.34.65.183,5000,,0");
			addServer("asss test zone", "???");
			
 			return;
		}
		BufferedReader file = new BufferedReader( in );
		
		String line="";
		String[] part;
		int serverCount = 0;
		boolean bLastLineWasServer = false;
		while(true)
		{
			try
			{
				if (file.ready())
					line = file.readLine().trim();
				else
					break;
			}
			catch(IOException e)
			{
				lm.Log("E", "<zoneman> " + e.getMessage());
				break;
			}

			if (line.length() < 2) continue;
			if (line.startsWith("#") && bLastLineWasServer)
			{
				descriptionTable.put("" + (serverCount-1),
					line.substring(1));
				bLastLineWasServer = false;
			}
			else
			{
   				serverTable.put("" + serverCount, line);
				bLastLineWasServer = true;
				
				part = line.split(",", 5);
				if (part.length >= 3)
				{
					addServer(part[0], "???");
					if (bPingNow)
					{
						Runnable pingLater =
							new pingLater(serverCount);
						SwingUtilities.invokeLater(pingLater);
					}
				}
				
				serverCount++;
			}
		}
	}

	class pingLater implements Runnable
	{
		int index;
	
		public pingLater(int index)
		{
			this.index = index;
		}
	
		public void run()
		{
			//Get info for the selected server
			if (index < 0) return;
			String info = (String)serverTable.get("" + index);
			if (info == null) return; //null
			String[] part = info.split(",", 5);
			
			String address =  part[1];
			int    port    = Integer.parseInt( part[2] ) + 1;
		
			//Ping
			ping pinger = new ping(lm);			
			int num = pinger.pingServer(address, port);
			pinger.finish();
			
			String playercount = "" + num;
			if (num < 0) playercount = "[DOWN]";
			popListModel.set(index, playercount);
		}
	}

	public void writeProfile(String fname)
	{
		PrintWriter out;
		
		try
		{
			out = new PrintWriter(
				new FileOutputStream("zone.dat", false)
				);
		}
		catch(Exception e)
		{
			e.printStackTrace();
			return;
		}
		
		int i, count = serverTable.size();
		String server, info;
		for(i=0; i<count; i++)
		{
			server = (String)serverTable.get("" + i);
			info   = (String)descriptionTable.get("" + i);
			if (server == null)
			{
				count++;
				continue;
			}
			
			//Write server
			out.println(server);
			
			if (info != null)
			{
				out.println("#" + info);
			}
		}
		
		out.close();
	}

	//Move to config.class as UpdateStr()?
	private void updateKey(PrintWriter out, String section, String key,
		String value)
	{
		if (value == null) return;
		String orig = cfg.GetStr(section, key);
		if (orig == null) return;
		if (!orig.equals(value))
		{
			out.println(section + ":" + key + "=" + value);
		}
	}

	public void writeConfig(String fname)
	{
		PrintWriter out;
		
		try
		{
			out = new PrintWriter(
				new FileOutputStream(fname, true)
				);
		}
		catch(Exception e)
		{
			e.printStackTrace();
			return;
		}
		
		updateKey(out, "ChatNet", "Name", nameField.getText());
		updateKey(out, "ChatNet", "Pass",
			new String(passField.getPassword()) );
		updateKey(out, "ChatNet", "Auto", autoField.getText());

		out.close();
	}
}
